/*

 (C) COPYRIGHT 2006, 
 Julin Simn de Castro, simon@losInvisibles.net
 http://calconsole.losInvisibles.net

 This program is free software; you can redistribute it and/or modify
 it under the terms of either: 

 The GPL as published by the Free Software Foundation,    
 version 2 or version 3, available at: http://www.fsf.org/copyleft/lesser.html

*/

#include "libical/ical.h"
#include "libical/icalss.h"
#include "libical/icalfileset.h"
#include "libical/icalset.h"
#include "calendarics.h"
#include "calendar.h"
#include <string.h>

struct date_list_link* get_days_events(char* d_ini, char* d_end, char* file_cal) {

    icalset *cout;
    icalcomponent *itr;
    icalsetiter iter;

    struct icaltimetype dtstart_user, dtend_user; // Date periody choose user for show
    struct icaltimetype dtstart_cal; // Date ini in calendar
    struct icaltimetype dt_comp; 
    struct date_list_link *date_list, *date_list_init = NULL, *pointer, *before;
    int count =  0;
    int i;
    icalcompiter ievent;

    if (LOGS) 
        printf("get_days_events()\n");

    if (VERBOSE || LOGS) {
        printf("\nDate_ini: %s\n", d_ini); 
        printf("Date_end: %s\n", d_end);
    } 

    dtstart_user = icaltime_from_string(d_ini);
    dtend_user = icaltime_from_string(d_end);

    cout = icalfileset_new(file_cal);


    /* Loop for components of calendars */

    for (itr = icalfileset_get_first_component(cout);
         itr != 0; 
         itr = icalfileset_get_next_component(cout)) {

        /* List all of the VEVENTS */

        for(ievent = icalcomponent_begin_component(itr,ICAL_VEVENT_COMPONENT);
            icalcompiter_deref(&ievent)!= 0; icalcompiter_next(&ievent)){
    
            icalcomponent *the_event = icalcompiter_deref(&ievent);

            count++;

            dtstart_cal = icalcomponent_get_dtstart(the_event);

            /* Date for compare */
            dt_comp = dtstart_user;

            /* Add day to the date end*/
            icaltime_adjust(&dtend_user, 1, 0, 0, 0);

            /* Loop for periody of days */
            while(icaltime_compare_date_only(dt_comp, dtend_user) != 0) {

                /* Compare date calendar with date periody */
                if(icaltime_compare_date_only(dtstart_cal, dt_comp) == 0) {

                    if (date_list_init == NULL) {
                        date_list_init = (struct date_list_link *)malloc(sizeof(struct date_list_link));

                        if (date_list_init == NULL){
                            printf(NOMEMORY);
                            exit (-1);
                        }

                        date_list_init->year = dtstart_cal.year;
                        date_list_init->month = dtstart_cal.month;
                        date_list_init->day = dtstart_cal.day;
                        date_list_init->next = NULL;
                        before = date_list_init;
                    }
                    else {
                        pointer = (struct  date_list_link*)malloc(sizeof(struct date_list_link));

                        if (pointer == NULL) {
                            printf(NOMEMORY);
                            exit (-1);
                        }

                        pointer->year = dtstart_cal.year;
                        pointer->month = dtstart_cal.month;
                        pointer->day = dtstart_cal.day;
                        before->next = pointer;
                        pointer->next = NULL;
                        before = pointer;
                    }
                }

                /* Add day */
                icaltime_adjust(&dt_comp, 1, 0, 0, 0);
            }
        }
    }

    icalset_free(cout);

   return date_list_init;
}

int is_day_events(struct date_list_link* dates, int j) {

    struct date_list_link *the_pointer;
    struct date *the_date;

    if (LOGS)
        printf("is_day_events()\n");

    the_pointer = dates;

    while (the_pointer != NULL) {

        the_date = (struct date*)malloc(sizeof(struct date));

        if (the_date == NULL) {
            printf(NOMEMORY);
            exit (-1);
        }

        the_date->y = the_pointer->year;
        the_date->m = the_pointer->month;
        the_date->d = the_pointer->day;

        if (sndays(the_date) == j) { 
            return 1;
        }

        the_pointer = the_pointer->next;
    
        free(the_date);
    }

    return 0;
}


void get_events_day(char* the_date, char* the_file) {

    icalset *cout;
    icalcomponent *itr;
    icalcompiter ievent;

    struct icaltimetype dtstart_cal; // Date ini in calendar
    struct icaltimetype dt_day; 
    struct evento *event_day = NULL;
    int count = 0, num_elem = 0;
    int i;

    if (LOGS)
        printf("get_events_day()\n"); 

    if (VERBOSE || LOGS) {
        printf("\nDate: %s\n", the_date); 
        printf("File: %s\n", the_file);
    } 

    event_day = (struct evento *) malloc(sizeof(struct evento));
            
    if (event_day == NULL) {
        printf(NOMEMORY);
        exit (-1);
    }
    event_day->next = NULL;

    dt_day = icaltime_from_string(the_date);

    cout = icalfileset_new(the_file);

    /* Loop for events of calendars */
    for (itr = icalfileset_get_first_component(cout);
         itr != 0; 
         itr = icalfileset_get_next_component(cout)) {


        /* List all of the VEVENTS */

        for(ievent = icalcomponent_begin_component(itr,ICAL_VEVENT_COMPONENT);
            icalcompiter_deref(&ievent)!= 0; icalcompiter_next(&ievent)){
    
            icalcomponent *the_event = icalcompiter_deref(&ievent);

            count++;

            dtstart_cal = icalcomponent_get_dtstart(the_event);

            if (LOGS)
                printf("dtstart_cal: %s\n", icaltime_as_ical_string(dtstart_cal));

            if (icaltime_compare_date_only(dtstart_cal, dt_day) == 0) {
                
                char* the_time;
                char *the_description, *the_uid, *the_summary, *the_location;
                int itime;

                num_elem++;

                the_time = (char*) malloc(sizeof(char*) * 6);

                if (the_time == NULL) {
                    printf(NOMEMORY);
                    exit (-1);
                }

//printf("1\n");

                sprintf(the_time, "%02d%02d", dtstart_cal.hour, dtstart_cal.minute);
                itime = atoi(the_time);

                the_description = (char*)icalcomponent_get_description(the_event);
                the_uid = (char*)icalcomponent_get_uid(the_event);
                the_summary = (char*)icalcomponent_get_summary(the_event);
                the_location = (char*)icalcomponent_get_location(the_event);

                sprintf(the_time, "%02d:%02d", dtstart_cal.hour, dtstart_cal.minute);

//printf("2\n");
                if (VERBOSE || LOGS) 
                    printf("Time: %s\n", the_time); 

//printf("3\n");
                /* Insert sort into list linked */
                if (num_elem == 1) {
                    event_day->time = itime;
                    event_day->hour_minute = the_time;
                    event_day->description = the_description;
                    event_day->uid = the_uid;
                    event_day->summary = the_summary;
                    event_day->location = the_location;
                }
                else {
                    insert_event(&event_day, itime, the_time, the_description, the_uid, 
                                the_summary, the_location);
                }
            }
        }
    }

    /*  list linked */

//printf("4\n");
    struct evento *the_pointer = NULL, *the_before = NULL;

    the_pointer = event_day;

    while (the_pointer != NULL) {

        if (the_pointer->description != NULL || the_pointer->summary != NULL) {
            printf("\n");
            printf(INVERSE);
            printf("%s", the_pointer->hour_minute);
            printf(PLAIN);
            printf("\n");
            printf("   %s\n", the_pointer->description);
            //printf("UID: %s\n", the_pointer->uid);
            printf("   %s\n", the_pointer->summary);
            //printf("Location: %s\n", the_pointer->location);
            printf(SEPARATOREVENT);
        }

        the_before = the_pointer;
        the_pointer = the_pointer->next;
        free(the_before);
   }
   printf("\n");

    //free(event_day);
}

/* Linked list sort */
void insert_event(struct evento **L, int elem, char* the_time, char* the_description, 
                char* the_uid, char* the_summary, char* the_location) {

    struct evento *actual, *anterior, *nuevo;

    if (LOGS)
        printf("insert_event()\n");

    if (VERBOSE || LOGS) {
        printf("\nTime: %s\n", the_time); 
        printf("Description: %s\n", the_description); 
        printf("Summary: %s\n", the_summary); 
        printf("Location: %s\n", the_location); 
        printf("UID: %s\n", the_uid); 
    }

    /* 1.- Look for his position */
    anterior = actual = *L;

    while (actual != NULL && actual->time < elem) {
        anterior = actual;
        actual = actual->next;
    }

    /* 2.- Make the nodo */
    nuevo = (struct evento *) malloc(sizeof(struct evento));
    nuevo->time = elem;
    nuevo->hour_minute= the_time;
    nuevo->description = the_description;
    nuevo->uid = the_uid;
    nuevo->summary = the_summary;
    nuevo->location = the_location;

    /* 3.- Link */
    if (anterior == NULL || anterior == actual) { /* insert to first */

        nuevo->next = anterior;
        *L = nuevo;  /* to insert the first update the head */
    }
    else {  /* insert into middle or end */
        nuevo->next = actual;
        anterior->next = nuevo;
    }
}

size_t chrsubst(char *s1, int ch, int ch2) {

     size_t count = 0;                /* The count to return */
     char   *wrk = strchr(s1, ch);    /* Find first char in s1 */

    if (LOGS)
        printf("chrsubst()\n");

     while (wrk) {                    /* While we have matches */
         *wrk = (char) ch2;           /* Replace the character */
         count++, wrk++;              /* Increment the count & pointer */
         wrk = strchr(wrk, ch);       /* Search for next occurance */
     }
     return count;                    /* Return the count */

} 

/* Return the path/file of the file calendar */
void get_path_cal (char* the_file) {

    char* file;
    char* home = (char*)getenv ("HOME"); 
    const char* delimiter = "/";
    const char* path_local = "./";

    if (LOGS)
        printf("get_path_cal()\n");

    if (home == NULL) 
        strcpy(home, path_local);

    if (!strcmp(the_file, NOFILE)) { 
        file = (char*)strcat(home, FILE_CAL);
        strcpy (the_file, file);
    }
}

/* Return the path/file of the file zoneinfo and zones.tab */
void get_zoneinfo (char* the_file) {

    if (LOGS)
        printf("get_zoneinfo()\n");

    if (fopen(LOCALSHAREZONESTAB, "r") != NULL) 
        strcpy (the_file, LOCALSHAREZONES);
    else if (fopen(SHAREZONESTAB, "r") != NULL) 
        strcpy (the_file, SHAREZONES);
    else if (fopen(ETCZONESTAB, "r") != NULL) 
        strcpy (the_file, ETCZONES);
    else
        strcpy (the_file, ZONEINFODEFAULT);

    if (LOGS)
        printf("File zoneinfo: %s\n", the_file);
}


void create_calendar(char *the_title, char* the_summary, char *the_hour_ini, 
                    char *the_hour_end, char* the_day, char* file_cal, char* the_locale) {

    icalset *cout;
    icalcomponent* calendar;
    icalcomponent* event;
    struct icaltimetype start, end;
    icalproperty* property;
    char *calendar_as_string;
    char **cad;
    char *the_date;
    static char  *file_zone = NULL; /* File zoneinfo zones.tab  */

    if (LOGS) {
        printf("create_calendar()\n");
        if (!strcmp(the_day, "")) 
            printf("the_day %s\n", the_day);
    }

    file_zone = (char*)malloc(sizeof(char) * FILE_SIZE);

    get_zoneinfo(file_zone);
    set_zone_directory(file_zone);
    putenv(TZ);

    the_date = (char*)malloc(sizeof(char) * DATE_SIZE);

    cout = icalfileset_new(file_cal);

    /* The time now */
    start = icaltime_from_timet(time(0),0);
    end = icaltime_from_timet(time(0),0);

    /* The day */
    if (strcmp(the_day, "")) {

        chrsubst(the_day, ':', '/');
        chrsubst(the_day, '-', '/');
        chrsubst(the_day, ' ', '/');
        chrsubst(the_day, '.', '/');

        if (the_locale != NULL && !(strcmp(the_locale, "ES"))) {
            start.day = atoi(strtok_r(the_day, DELIM_DATE, (char**)&cad));
            start.month = atoi(strtok_r((char*)cad, DELIM_DATE, &the_date));
        }
        else {
            start.month = atoi(strtok_r(the_day, DELIM_DATE, (char**)&cad));
            start.day = atoi(strtok_r((char*)cad, DELIM_DATE, &the_date));
        }
        start.year = atoi(the_date);
    }

    /* The hour initial */
    if (strcmp(the_hour_ini, "")) {
        chrsubst(the_hour_ini, '/', ':');
        chrsubst(the_hour_ini, '-', ':');
        chrsubst(the_hour_ini, ' ', ':');
        chrsubst(the_hour_ini, '.', ':');

        start.hour = atoi(strtok_r(the_hour_ini, DELIM_HOUR, (char**)&cad));
        start.minute = atoi(strtok_r((char*)cad, DELIM_HOUR, &the_date));
        start.second = 0;
    }

    /* The hour end */
    if (strcmp(the_hour_end, "")) {
        chrsubst(the_hour_end, '/', ':');
        chrsubst(the_hour_end, '-', ':');
        chrsubst(the_hour_end, ' ', ':');
        chrsubst(the_hour_end, '.', ':');

        end.hour = atoi(strtok_r(the_hour_end, DELIM_HOUR, (char**)&cad));
        end.minute = atoi(strtok_r((char*)cad, DELIM_HOUR, &the_date));
        end.second = 0;
    }
    else {
        end = start;
        end.hour++;  
    }

    if (LOGS) {
        printf("the_day %s\n", the_day);
        printf("the_hour_ini %s\n", the_hour_ini);
        printf("the_hour_end %s\n", the_hour_end);

    }
    /* Create calendar and add properties */
    calendar = icalcomponent_new(ICAL_VCALENDAR_COMPONENT);

    icalcomponent_add_property(calendar, icalproperty_new_prodid(PROIDCALCONSOLE));

    /* Add an event */
    event = icalcomponent_new(ICAL_VEVENT_COMPONENT);

    /* add a property that has parameters */
    //icalcomponent_add_property(calendar, icalproperty_new_version("7.0"));
    //icalproperty_add_parameter(property,icalparameter_new_role(ICAL_ROLE_CHAIR));
    //property = icalproperty_new_organizer("simon@losInvisibles.net");
    //icalcomponent_add_property(event,property);
    //icalcomponent_add_property(event,icalproperty_new_categories("MEETING"));
    //icalcomponent_add_property(event,icalproperty_new_class(ICAL_CLASS_PRIVATE));

    if (strcmp(the_title, ""))
        icalcomponent_add_property(event,icalproperty_new_description(the_title));

    if (strcmp(the_summary, ""))
        icalcomponent_add_property(event,icalproperty_new_summary(the_summary));
                                                                                       
    icalcomponent_add_property(event, icalproperty_new_created(icaltime_from_timet(time(0),0)));

    property = icalproperty_new_dtstart(start);
    icalcomponent_add_property(event, property);

    property = icalproperty_new_dtend(end);
    icalcomponent_add_property(event, property);

    icalcomponent_add_component(calendar,event);

    calendar_as_string = icalcomponent_as_ical_string(calendar);

    icalfileset_add_component(cout, calendar);
    icalfileset_commit(cout);

    icalcomponent_free(calendar);
    icalset_free(cout);
}


int remove_day(char* the_day, char* the_file, char* the_locale) {

    icalset *cout;
    icalcomponent *itr;

    struct icaltimetype dtstart_cal; // Date ini in calendar
    struct icaltimetype dt_day; 
    int count = 0;
    char **cad;
    char *the_date;
    static char  *file_zone = NULL; /* File zoneinfo zones.tab  */
    struct icaltimetype start;

    if (LOGS) {
        printf("remove_day()\n"); 
    }

    the_date = (char*)malloc(sizeof(char) * DATE_SIZE);
    file_zone = (char*)malloc(sizeof(char) * FILE_SIZE);

    get_zoneinfo(file_zone);
    set_zone_directory(file_zone);
    putenv(TZ);

    /* The day */
    chrsubst(the_day, ':', '/');
    chrsubst(the_day, '-', '/');
    chrsubst(the_day, ' ', '/');
    chrsubst(the_day, '.', '/');

    if (the_locale != NULL && !(strcmp(the_locale, "ES"))) {
        dt_day.day = atoi(strtok_r(the_day, DELIM_DATE, (char**)&cad));
        dt_day.month = atoi(strtok_r((char*)cad, DELIM_DATE, &the_date));
    }
    else {
        dt_day.month = atoi(strtok_r(the_day, DELIM_DATE, (char**)&cad));
        dt_day.day = atoi(strtok_r((char*)cad, DELIM_DATE, &the_date));
    }

    dt_day.year = atoi(the_date);

    cout = icalfileset_new(the_file);

    /* Loop for events of calendars */
    for (itr = icalfileset_get_first_component(cout);
         itr != 0; 
         itr = icalfileset_get_next_component(cout)) {

        count++;

        icalcomponent *the_event;

        the_event = icalcomponent_get_first_component(itr,ICAL_VEVENT_COMPONENT);
        dtstart_cal = icalcomponent_get_dtstart(the_event);

        if (icaltime_compare_date_only(dtstart_cal, dt_day) == 0) 
            icalfileset_remove_component(cout, itr);
    }

    icalset_free(cout);
}

/* Show events of today */
void printtoday(char* day, char* the_locale) {

    struct icaltimetype the_today;

    if (LOGS) 
        printf("printtoday()\n");
 
    the_today = icaltime_from_timet(time(0),0);

    if (the_locale != NULL && !(strcmp(the_locale, "ES"))) 
        sprintf(day, "%02d/%02d/%02d", the_today.day, the_today.month, the_today.year);
    else 
        sprintf(day, "%02d/%02d/%02d", the_today.month, the_today.day, the_today.year);
        
    printday (day, the_locale);
}
